@@ -392,6 +392,11 @@ public class TestDatabaseConfiguration {
         assertFalse(it.hasNext());
     }
 
+    @Test
+    public void testGetKeysInternalNoDatasource() throws Exception {
+        ConfigurationUtils.toString(new DatabaseConfiguration());
+    }
+
     @Test
     public void testGetKeysMultiple() throws ConfigurationException {
         final Configuration config = helper.setUpMultiConfig();
@@ -509,4 +514,5 @@ public class TestDatabaseConfiguration {
         final String[] values = config.getStringArray("keyList");
         assertArrayEquals(new String[] {"1", "2", "3"}, values);
     }
+
 }
